# came from: http://stackoverflow.com/questions/6811902/import-arbitrary-named-file-as-a-python-module-without-generating-bytecode-file
import sys
import imp
import contextlib


@contextlib.contextmanager
def preserve_value(namespace, name):
    """ A context manager to preserve, then restore, the specified binding.

        :param namespace: The namespace object (e.g. a class or dict)
            containing the name binding.
        :param name: The name of the binding to be preserved.
        :yield: None.

        When the context manager is entered, the current value bound to `name`
        in `namespace` is saved. When the context manager is exited, the
        binding is re-established to the saved value.

        """
    saved_value = getattr(namespace, name)
    yield
    setattr(namespace, name, saved_value)


def make_module_from_file(module_name, module_filepath):
    """ Make a new module object from the source code in specified file.

        :param module_name: The name of the resulting module object.
        :param module_filepath: The filesystem path to open for
            reading the module's Python source.
        :return: The module object.

        The Python import mechanism is not used. No cached bytecode
        file is created, and no entry is placed in `sys.modules`.

        """
    py_source_open_mode = 'U'
    py_source_description = (b".py", py_source_open_mode, imp.PY_SOURCE)

    with open(module_filepath, py_source_open_mode) as module_file:
        with preserve_value(sys, 'dont_write_bytecode'):
            sys.dont_write_bytecode = True
            module = imp.load_module(
                module_name,
                module_file,
                module_filepath,
                py_source_description)

    return module



import os
import sys
from os.path import abspath, join, dirname, expanduser
import logging
import logging.config

from tenyks.module_loader import make_module_from_file

logger = logging.getLogger('tenyks')

PROJECT_ROOT = abspath(dirname(__file__))


class NotConfigured(Exception):
    pass


# taken from legit (https://github.com/kennethreitz/legit)
class Settings(object):
    _singleton = {}

    # attributes with defaults
    __attrs__ = tuple()

    def __init__(self, **kwargs):
        super(Settings, self).__init__()

        self.__dict__ = self._singleton

    def __call__(self, *args, **kwargs):
        # new instance of class to call
        r = self.__class__()

        # cache previous settings for __exit__
        r.__cache = self.__dict__.copy()
        map(self.__cache.setdefault, self.__attrs__)

        # set new settings
        self.__dict__.update(*args, **kwargs)

        return r

    def __enter__(self):
        pass

    def __exit__(self, *args):

        # restore cached copy
        self.__dict__.update(self.__cache.copy())
        del self.__cache

    def __getattribute__(self, key):
        if key in object.__getattribute__(self, '__attrs__'):
            try:
                return object.__getattribute__(self, key)
            except AttributeError:
                return None
        return object.__getattribute__(self, key)


settings = Settings()


def collect_settings(settings_path=None):
    intrl_settings = None
    if not settings_path:
        if len(sys.argv) > 1:
            intrl_settings = make_module_from_file('settings', sys.argv[1])
        else:
            path = join(expanduser('~'), '.config', 'tenyks', 'settings.py')
            if os.path.exists(path):
                intrl_settings = make_module_from_file('settings', path)
            else:
                path = join('etc', 'tenyks', 'settings.py')
                if os.path.exists(path):
                    intrl_settings = make_module_from_file('settings', path)
    else:
        if os.path.exists(settings_path):
            intrl_settings = make_module_from_file('settings', settings_path)

    if not intrl_settings:
        message = """
You need to provide a settings module.

Use `tenyksmkconfig > /path/to/settings.py` and run Tenyks with
`tenyks /path/to/settings.py` after you edit it accordingly.
        """.format(pr=PROJECT_ROOT)
        raise NotConfigured(message)

    for sett in filter(lambda x: not x.startswith('__'), dir(intrl_settings)):
        setattr(settings, sett, getattr(intrl_settings, sett))

    if not hasattr(intrl_settings, 'WORKING_DIR'):
        WORKING_DIR = getattr(settings, 'WORKING_DIRECTORY_PATH',
                join(os.environ['HOME'], '.config', 'tenyks'))
        DATA_WORKING_DIR = join(WORKING_DIR, 'data')

        setattr(settings, 'WORKING_DIR', WORKING_DIR)
        setattr(settings, 'DATA_WORKING_DIR', DATA_WORKING_DIR)

    LOGGING_CONFIG = {
        'version': 1,
        'disable_existing_loggers': True,
        'formatters': {
            'color': {
                'class': 'tenyks.logs.ColorFormatter',
                'format': '%(asctime)s %(name)s:%(levelname)s %(message)s'
            },
            'default': {
                'format': '%(asctime)s %(name)s:%(levelname)s %(message)s'
            }
        },
        'handlers': {
            'console': {
                'level': 'DEBUG',
                'class': 'logging.StreamHandler',
                'formatter': 'color'
            },
            'file': {
                'level': 'INFO',
                'class': 'logging.FileHandler',
                'formatter': 'default',
                'filename': join(WORKING_DIR, 'tenyks.log')
            }
        },
        'loggers': {
            'tenyks': {
                'handlers': ['console'],
                'level': ('DEBUG' if settings.DEBUG else 'INFO'),
                'propagate': True
            },
        }
    }

    logging.config.dictConfig(LOGGING_CONFIG)

def make_config():
    with open(join(PROJECT_ROOT, 'settings.py.dist'), 'r') as f:
        for line in f.readlines():
            print line,




#!/usr/bin/env python
from setuptools import setup, find_packages
import sys, os

version = '0.1.22'

setup(name='tenyks',
      version=version,
      description="Redis powered IRC bot",
      long_description="""\
""",
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='irc bot redis',
      author='Kyle Terry',
      author_email='kyle@kyleterry.com',
      url='https://github.com/kyleterry/tenyks',
      license='LICENSE',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      package_dir={'tenyks': 'tenyks'},
      package_data={'tenyks': ['logging_config.ini']},
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'gevent',
          'redis',
          'nose',
          'unittest2',
          'clint',
      ],
      entry_points={
          'console_scripts': [
              'tenyks = tenyks.core:main',
              'tenyksmkconfig = tenyks.config:make_config'
          ]
      },
      )



""" Core of tenyks. Contains Robot and IRC/Redis Lines"""
import gevent.monkey
gevent.monkey.patch_all()

from datetime import datetime
import hashlib
import json
import os
import re
import sys
import time

import logging
logger = logging.getLogger('tenyks')

import gevent
from gevent import queue
import redis

from tenyks.config import settings, collect_settings
from tenyks.connection import Connection
from tenyks.utils import pubsub_factory, parse_irc_message, get_privmsg_data
from tenyks.middleware import CORE_MIDDLEWARE


if hasattr(settings, 'MIDDLEWARE'):
    CORE_MIDDLEWARE += settings.MIDDLEWARE


class Robot(object):
    """
    Design Rules:

    * The Robot has access to services to handle commands.
    * Services send and receive data.
    * The Robot's brain is, effectively, the services.
    * The Robot will forever ONLY relay data to and from platforms (IRC, Email)
      to and from services and handle connection to platforms.
    * The Robot will never handle commands sent from platforms or services.
    * _Almost_ everything is evented.
    """

    def __init__(self):
        self.broadcast_queue = queue.Queue()
        gevent.spawn(self.broadcast_loop)
        gevent.spawn(self.handle_incoming_redis_messages)

        self.prepare_environment()

        self.connections = {}
        self.bootstrap_connections()

    def prepare_environment(self):
        # TODO FIX THIS MESS
        try:
            os.mkdir(settings.WORKING_DIR)
        except OSError:
            # Already exists
            pass
        try:
            os.mkdir(settings.DATA_WORKING_DIR)
        except OSError:
            # Already exists
            pass

    def bootstrap_connections(self):
        for name, connection in settings.CONNECTIONS.iteritems():
            conn = Connection(name, **connection)
            conn.connect()
            self.connections[name] = conn
            self.set_nick_and_join(conn)

    def set_nick_and_join(self, connection):
        self.send(connection.name, 'NICK {nick}'.format(
            nick=connection.config['nick']))
        self.send(connection.name, 'USER {ident} {host} bla :{realname}'.format(
            ident=connection.config['ident'],
            host=connection.config['host'],
            realname=connection.config['realname']))

        # join channels
        for channel in connection.config['channels']:
            self.join(channel, connection)

    def join(self, channel, connection, message=None):
        """ join a irc channel
        """
        password = ''
        if ',' in channel:
            channel, password = channel.split(',')
        chan = '{channel} {password}'.format(
                channel=channel, password=password)
        self.send(connection.name, 'JOIN {channel}'.format(
            channel=chan.strip()))

    def handle_irc_ping(self, connection, message):
        """
        always returns None
        """
        connection.last_ping = datetime.now()
        logger.debug(
            '{connection} Connection Worker: last_ping: {dt}'.format(
                connection=connection.name, dt=connection.last_ping))
        message = message.replace('PING', 'PONG')
        self.send(connection.name, message)

    def send(self, connection, message):
        """
        send a message to an IRC connection
        """
        message = message.strip()
        self.connections[connection].output_queue.put(message)
        logger.info('Robot -> {connection}: {message}'.format(
            connection=connection, message=message))

    def say(self, connection, message, channels=[]):
        for channel in channels:
            message = 'PRIVMSG {channel} :{message}\r\n'.format(
                    channel=channel, message=message)
            self.send(connection, message)

    def broadcast_loop(self):
        """
        Pop things off the broadcast_queue and create jobs for them.
        """
        while True:
            data = self.broadcast_queue.get()
            gevent.spawn(self.broadcast_worker, data)

    def broadcast_worker(self, data):
        """
        This worker will broadcast a message to the service broadcast channel
        """
        r = redis.Redis(**settings.REDIS_CONNECTION)
        broadcast_channel = getattr(settings, 'BROADCAST_TO_SERVICES_CHANNEL',
            'tenyks.services.broadcast_to')
        r.publish(broadcast_channel, json.dumps(data))

    def handle_incoming_redis_messages(self):
        """
        {
            'payload': 'this is a message to IRC',
            'target': '#test',
            'command': 'PRIVMSG',
            'connection': 'freenode',
        }
        """
        broadcast_channel = getattr(settings, 'BROADCAST_TO_ROBOT_CHANNEL',
            'tenyks.robot.broadcast_to')
        pubsub = pubsub_factory(broadcast_channel)
        for raw_redis_message in pubsub.listen():
            logger.info('Robot <- {data}'.format(
                data=json.dumps(raw_redis_message)))
            try:
                if raw_redis_message['data'] != 1L:
                    message = json.loads(raw_redis_message['data'])
                    if message['command'].lower() == 'privmsg':
                        self.say(message['connection'],
                                    message['payload'],
                                    channels=[message['target']])
            except ValueError:
                logger.info('Robot Pubsub: invalid JSON. Ignoring message.')

    def connection_worker(self, connection):
        while True:
            if connection.user_disconnect:
                break
            needs_reconnect = connection.needs_reconnect
            ping_delta = datetime.now() - connection.last_ping
            no_ping = ping_delta.seconds > 5 * 60
            if needs_reconnect or no_ping:
                connection.reconnect()
                self.set_nick_and_join(connection)
            try:
                raw_line = connection.input_queue.get(timeout=5)
            except queue.Empty:
                continue
            if raw_line.startswith('PING'):
                self.handle_irc_ping(connection, raw_line)
                continue
            else:
                data = get_privmsg_data(
                    connection, *parse_irc_message(raw_line))
                if data:
                    for middleware in CORE_MIDDLEWARE:
                        data = middleware(connection, data)
                    self.broadcast_queue.put(data)
        logger.info('{connection} Connection Worker: worker shutdown'.format(
            connection=connection.name))

    def run(self):
        try:
            while True:
                self.workers = []
                for name, connection in self.connections.iteritems():
                    self.workers.append(
                            gevent.spawn(self.connection_worker, connection))
                gevent.joinall(self.workers)
                break
        except KeyboardInterrupt:
            logger.info('Robot: shutting down: user disconnect')
            for name, connection in self.connections.iteritems():
                connection.close()
        finally:
            for name, connection in self.connections.iteritems():
                self.send(connection.name,
                        'QUIT :{message}'.format(
                            message=getattr(self, 'exit_message', 'I\'m out!')))
                connection.close()
            sys.exit('Bye.')


def main():
    collect_settings()
    robot = Robot()
    robot.run()


if __name__ == '__main__':
    main()



import re

from redis import Redis

from tenyks.config import settings

def pubsub_factory(channel):
    """
    Returns a Redis.pubsub object subscribed to `channel`.
    """
    rdb = Redis(**settings.REDIS_CONNECTION)
    pubsub = rdb.pubsub()
    pubsub.subscribe(channel)
    return pubsub


def parse_irc_message(s):
    prefix = ''
    trailing = []
    if not s:
        return None
    if s[0] == ':':
        prefix, s = s[1:].split(' ', 1)
    if s.find(' :') != -1:
        s, trailing = s.split(' :', 1)
        args = s.split()
        args.append(trailing)
    else:
        args = s.split()
    command = args.pop(0)
    return prefix, command, args


irc_prefix_re = re.compile(r':?([^!@]*)!?([^@]*)@?(.*)').match
def parse_irc_prefix(prefix):
     nick, user, host = irc_prefix_re(prefix).groups()
     return nick, user, host


def parse_args(args):
    target, message = args
    return target, message


def get_privmsg_data(connection, prefix, command, args):
    data = {}
    if command == 'PRIVMSG':
        nick, user, host = parse_irc_prefix(prefix)
        target, message = parse_args(args)
        data['nick'] = nick
        data['user'] = user
        data['host'] = host
        data['mask'] = '{user}@{host}'.format(user=user, host=host)
        data['command'] = command
        data['connection'] = connection.name
        data['target'] = target
        if target.startswith('#'):
            data['from_channel'] = True
        else:
            data['from_channel'] = False
        data['full_message'] = message
        data['direct'] = (
                message.startswith(connection.config['nick']) or
                not data['from_channel'])
        if data['direct'] and data['from_channel']:
            data['payload'] = ' '.join(message.split()[1:])
        else:
            data['payload'] = data['full_message']
    return data
